#ifndef _TYPES
#define _TYPES

#include <math.h>

#define Cos(th) cos(3.1415926/180*(th))
#define Sin(th) sin(3.1415926/180*(th))

//
//  3D vector type
//
class Vec3
{
   public:
      double x,y,z;
      Vec3(void)
      {
         x = y = z = 0;
      }
      Vec3(double X,double Y,double Z)
      {
         x = X;
         y = Y;
         z = Z;
      }
};

//  Vec3 operators
inline Vec3   operator+(const Vec3& v1  , const Vec3& v2) {return Vec3(v1.x+v2.x , v1.y+v2.y , v1.z+v2.z);}
inline Vec3   operator-(const Vec3& v1  , const Vec3& v2) {return Vec3(v1.x-v2.x , v1.y-v2.y , v1.z-v2.z);}
inline double operator*(const Vec3& v1  , const Vec3& v2) {return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;}
inline Vec3   operator*(double f        , const Vec3& v)  {return Vec3(f*v.x , f*v.y , f*v.z);}
inline Vec3   operator*(const Vec3* rot , const Vec3& v)  {return Vec3(rot[0]*v , rot[1]*v , rot[2]*v);}

//  Normalize Vec3
inline Vec3 normalize(Vec3 v)
{
   double l = v*v;
   return (l==0) ? Vec3(1,0,0) : (1/sqrt(l))*v;
}

//
//  Ray type
//
class Ray
{
   public:
      Vec3 org;
      Vec3 dir;
      double t;
      Ray(Vec3 o,Vec3 d)
      {
         org = o;
         dir = d;
         t = 1e300;
      }
};

//
//  Color type
//
struct Color
{
   public:
      double r,g,b;
      Color()
      {
         r = g = b = 0;
      }
      Color(double R,double G,double B)
      {
         r = R;
         g = G;
         b = B;
      }
      Color& operator+=(const Color &c2) {r += c2.r; g += c2.g; b += c2.b; return *this;}
};
//  Color operators
inline Color operator*(const Color& c1 , const Color& c2) {return Color(c1.r*c2.r , c1.g*c2.g , c1.b*c2.b);}
inline Color operator+(const Color& c1 , const Color& c2) {return Color(c1.r+c2.r , c1.g+c2.g , c1.b+c2.b);}
inline Color operator*(double f        , const Color& c)  {return Color(f*c.r , f*c.g , f*c.b);}

//
//  Material type
//
class Material
{
   public:
      Color col;
      double reflection;
      Material()
      {
         reflection = 0;
      }
      Material(double r,double g,double b,double f)
      {
         col = Color(r,g,b);
         reflection = f;
      }
};

//
//  Light type
//
class Light
{
   public:
      Vec3  pos;
      Color col;
      Light(double x,double y,double z , double r,double g,double b)
      {
         pos = Vec3(x,y,z);
         col = Color(r,g,b);
      }
};

#endif
